//format string as title case
function titleCase(str) {
  return str.toLowerCase().split(' ').map(function(word) {
    return (word.charAt(0).toUpperCase() + word.slice(1));
  }).join(' ');
}


//send an email from an email log record
function emailPerson(emailLogKey){

  var emailRecord = app.models.EmailLog.getRecord(emailLogKey);
  var subject = emailRecord.Subject;
  var body = emailRecord.Body;
  var email = emailRecord.Person.Email;
  var options = {htmlBody: body};

  GmailApp.sendEmail(email,subject,body,options);

}


//generates a list of fields from the database for configuring email templates
function createFieldList(){
  var people = app.metadata.models.Person.fields;
  var peopleList = [];
  var value;
  for (var key in people){
    value = people[key].name;
    if(value !== 'Id' && value.slice(-3) !== '_fk'){
       peopleList.push('Person::' + value);
    }
  }

  var company = app.metadata.models.Company.fields;
  var companyList = [];
  for (key in company){
    value = company[key].name;
    if(value !== 'Id' && value.slice(-3) !== '_fk'){
       companyList.push('Company::' + value);
    }
  }

  var event = app.metadata.models.Event.fields;
  var eventList = [];
  for (key in event){
    value = event[key].name;
    if(value !== 'Id' && value.slice(-3) !== '_fk'){
       eventList.push('Event::' + value);
    }
  }
  
  var user = app.metadata.models.Directory.fields;
  var userList = [];
  for (key in user){
    value = user[key].name;
    if(value !== 'Id' && value.slice(-3) !== '_fk'){
       userList.push('User::' + value);
    }
  }
  
  var venue = app.metadata.models.Vendor.fields;
  var venueList = [];
  for (key in venue){
    value = venue[key].name;
    if(value !== 'Id' && value.slice(-3) !== '_fk'){
       venueList.push('Venue::' + value);
    }
  }
  
  return peopleList.concat(companyList, eventList, userList, venueList);
}


//substitute the field names in an email template with the relevant field data
function substituteTemplate(text, eventKey, personKey){
  
  var userEmail = Session.getActiveUser().getEmail();
  var query = app.models.Directory.newQuery();
  query.filters.PrimaryEmail._equals = userEmail;
  var user = query.run()[0];
  var event = app.models.Event.getRecord(eventKey);
  var venue = event.Venue;
  
  var person = app.models.Person.getRecord(personKey);
  var records = {Person: person,
                 Company: person.Company,
                 Event: event,
                 User: user,
                 Venue: venue
                };

  var fieldList = createFieldList();

  for(var i = 0; i < fieldList.length; i++){
    var search = fieldList[i];
    var table = search.split('::').splice(0, 1);
    var field = search.split('::').splice(1, 2);
    try{
      var value = records[table][field];
      if(value instanceof Date){
        value = value.toLocaleDateString();
      }
      text = text.replace('<<' + search + '>>', value);
    }catch(error){
      if(error.name !== 'TypeError'){
        console.log(error);
      }
    }
  }
  return text; 
}


//send emails to a list of people, with the provided template
//using the provided event, substituting in the relevant data
function sendTemplateEmails(templateKey, eventKey, personKeyList){
  var templateRecord = app.models.EmailTemplate.getRecord(templateKey);
  var templateSubject = templateRecord.Subject;
  var templateBody = templateRecord.Body;
  var HTML =  templateRecord.HTML;

  var people = app.models.Person.getRecords(personKeyList);
  var date = new Date();
  var logs = [];

  for(var i = 0; i < people.length; i++){
    var person = people[i];

    var to = person.Email;
    if (to !== null){
      var subject = substituteTemplate(templateSubject, eventKey, person._key);
      var body = substituteTemplate(templateBody, eventKey, person._key);
      var aliases = GmailApp.getAliases();
      var options = HTML ? {htmlBody: body} : {};
      var log = app.models.EmailLog.newRecord();
      log.Body = body;
      log.Person = person;
      log.Date = date;
      log.Subject = subject;
      logs.push(log);

      GmailApp.sendEmail(to, subject, body, options);
    }
  }
  app.saveRecords(logs);
}


//import a google sheet of people into an event
function importPeopleToEvent(eventKey, sheet_url){

  var event = app.models.Event.getRecord(eventKey);
  
  var sheet = SpreadsheetApp.openByUrl(sheet_url);
  //select the columns and rows in the selected document that have data
  var range = sheet.getDataRange();

  var columns = [null];
  var col;

  //grab the first row of the import file (should be column headers)
  for(col = 1; col <= range.getNumColumns(); col++){
    columns.push(titleCase(range.getCell(1, col).getValue()).replace(' ', ''));
  }
  
  function filterNull(value){return value;}
  
  var columnCheck = columns.filter(filterNull);
  
  if(columnCheck.length === 0){
    return null;
  }

  var records = [];
  
  //import the file matching columns by the column header
  for(var row = 2; row <= range.getNumRows(); row++){
    
    var query;
    var company;
    var person;
    
    //pull the data from each column for the current row
    var firstName = columns.indexOf('FirstName') >= 0 ? range.getCell(row, columns.indexOf('FirstName')).getValue().toString() : null;
    var lastName = columns.indexOf('LastName') >= 0 ? range.getCell(row, columns.indexOf('LastName')).getValue().toString() : null;
    var companyName = columns.indexOf('Company') >= 0 ? range.getCell(row, columns.indexOf('Company')).getValue().toString() : null;
    var title = columns.indexOf('Title') >= 0 ? range.getCell(row, columns.indexOf('Title')).getValue().toString() : null;
    var phone = columns.indexOf('Phone') >= 0 ? range.getCell(row, columns.indexOf('Phone')).getValue().toString() : null;
    var email = columns.indexOf('Email') >= 0 ? range.getCell(row, columns.indexOf('Email')).getValue().toString() : null;
    var notes = columns.indexOf('Notes') >= 0 ? range.getCell(row, columns.indexOf('Notes')).getValue().toString() : null;
    
    var valueCheck = [firstName, lastName, companyName, title, phone, email, notes].filter(filterNull);
    
    //check if there is data in the row
    if(valueCheck.length === 0){
      continue;
    }
    
    //check if company exists
    if(companyName !== null){
      query = app.models.Company.newQuery();
      query.filters.Name._equals = companyName;
      company = query.run()[0];

      //if company doesn't exist, create it
      if(!company){
        company = app.models.Company.newRecord();
        company.Name = companyName;
        app.saveRecords([company]);
      }
    }
    
    //check if person exists
    if(firstName !== null || lastName !== null){
      query = app.models.Person.newQuery();
      query.filters.FirstName._equals = firstName;
      query.filters.LastName._equals = lastName;
      person = query.run()[0];
    }
    
    //if person doesn't exist, create them
    if(!person){
      person = app.models.Person.newRecord();
    }
    
    person.FirstName = firstName;
    person.LastName = lastName;
    person.Email = email;
    person.Phone = phone;
    person.Title = title;
    person.Notes = notes;
    person.Company = company;
    app.saveRecords([person]);
    
    //check if person has been added to event yet
    query = app.models.Invitee.newQuery();
    query.filters.Person._key._equals = person._key;
    query.filters.Event._key._equals = event._key;
    var invitee = query.run()[0];
    
    //if no invitee record, create it
    if(!invitee){
      invitee = app.models.Invitee.newRecord();
      invitee.Event = event;
      invitee.Person = person;
      app.saveRecords([invitee]);
    }
  }
}


function generatePDF(event_id){
  
  var event = app.models.Event.getRecord(event_id);
  var sessions = event.Session;
  
  
  
}




































