function toCurrency(num){
  if(num !== null){
    return '$' + num.toFixed(2).replace(/\d(?=(\d{3})+\.)/g, '$&,');
  }else{
    return null;
  }
}

function sendEmailMessage(to, subject, msg, attachments){
  var file_list = [];
  for (var i = 0; i < attachments.length; i++){
    var id = attachments[i];
    var file = DriveApp.getFileById(id).getBlob();
    file_list.push(file.setContentTypeFromExtension());
  }
  MailApp.sendEmail(to, subject, msg, {htmlBody: msg, attachments: file_list});
}


//substitue merge fields for their respective values in the template
//pass list of lists [['name','value],['name2','value2']]
function substituteFields(layoutPart, nameValuePairs){
  for(var i = 0; i < nameValuePairs.length; i++){
    layoutPart.replaceText('<<' + nameValuePairs[i][0] + '>>', nameValuePairs[i][1]);
  }
  return layoutPart;
}


// create a schedule for all sessions related to the event provided
//types = pdf, doc, email
function createSchedulePDF(event_id, type){
  
  var event = app.models.Event.getRecord(event_id);
  var sessions = event.Session;
  
  
  var templateList = DriveApp.getFilesByName('EventScheduleTemplate');
  var template;

  if(!templateList.hasNext()){
    var url = 'https://docs.google.com/document/d/1b4-4A9uuccZ_Xo6fof_dHg-L7dlWFfjxuz_kZ-cM1Cw/edit?usp=sharing';
    var copy_id = DocumentApp.openByUrl(url).getId();
    template = DriveApp.getFileById(copy_id).makeCopy('EventScheduleTemplate');
  }else{
    template = templateList.next(); 
  }
  
  
  
  var fileName = event.Name + ' Schedule';
  var copy = template.makeCopy(fileName);
  var tmpId = copy.getId();
  var tmpDoc = DocumentApp.openById(tmpId);
  
  var mergeFields = [];
  
  var current_date = new Date();
  
  mergeFields.push(['event_name', event.Name],
                   ['current_date', current_date.toDateString()]
                   );
  
  var numRows = sessions.length + 1;
  var numCols = 4;
  
  
  
  var body = tmpDoc.getBody();
  var header = tmpDoc.getHeader();
  var footer = tmpDoc.getFooter();
  
  var table = body.findText('<<session_name>>').getElement().getParent().getParent().getParent().getParent();
  
  substituteFields(header, mergeFields);
  substituteFields(body, mergeFields);
  substituteFields(footer, mergeFields);
  
  var templateRow = table.getRow(0);
  var fieldData;
  
  //create a row for each related session
  for(var i = 0; i < sessions.length; i++){
    var new_row = table.insertTableRow(1, templateRow.copy());
    var date = sessions[i].Date !== null ? sessions[i].Date.toLocaleDateString() : '';
    var duration = sessions[i].Duration !== null ? sessions[i].Duration.toString() : '';
    fieldData = [];
    fieldData.push(['session_name', sessions[i].Name],
                   ['date', date],
                   ['time', sessions[i].Time],
                   ['duration', duration]
                  );
    substituteFields(new_row, fieldData);
  }
  
  table.removeRow(0);
  
  tmpDoc.saveAndClose();
  
  
  var blob;
  if(type==='pdf'){
    blob = tmpDoc.getAs(MimeType.PDF);
    var bytes = blob.getBytes();
    Drive.Files.remove(tmpDoc.getId());
    return bytes;
    
  }else if(type === 'email'){
    blob = tmpDoc.getAs(MimeType.PDF);
    var pdfFile = DriveApp.createFile(blob).setName(fileName + '.pdf');
    Drive.Files.remove(tmpDoc.getId());
    sendEmailMessage(Session.getActiveUser().getEmail(), fileName, 'Here is your ' + fileName, [pdfFile.getId()]);
    Drive.Files.remove(pdfFile.getId());
    
  }else if(type === 'doc'){
    return tmpDoc.getUrl();
    
  }
  
} 